/*
 * Copyright (C) 2007 Junjiro Okajima
 *
 * This program, aufs is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/* $Id: vfsub.h,v 1.2 2007/03/05 02:20:55 sfjro Exp $ */

#ifndef __AUFS_VFSUB_H__
#define __AUFS_VFSUB_H__

#include <linux/fs.h>
//#include <linux/version.h>

#ifdef __KERNEL__

static inline ssize_t
vfsub_read(struct file *file, char __user *buf, size_t count, loff_t *ppos)
{
	ssize_t err;
	//lockdep_off();
	err = vfs_read(file, buf, count, ppos);
	//lockdep_on();
	return err;
}

static inline ssize_t
vfsub_write(struct file *file, const char __user *buf, size_t count,
	    loff_t *ppos)
{
	ssize_t err;
	lockdep_off();
	err = vfs_write(file, buf, count, ppos);
	lockdep_on();
	return err;
}

static inline loff_t vfsub_llseek(struct file *file, loff_t offset, int origin)
{
	loff_t err;
	lockdep_off();
	err = vfs_llseek(file, offset, origin);
	lockdep_on();
	return err;
}

static inline int vfsub_unlink(struct inode *dir, struct dentry *dentry)
{
	int err;
	lockdep_off();
	err = vfs_unlink(dir, dentry);
	lockdep_on();
	return err;
}

static inline int vfsub_readdir(struct file *file, filldir_t filldir, void *arg)
{
	int err;
	lockdep_off();
	err = vfs_readdir(file, filldir, arg);
	lockdep_on();
	return err;
}

static inline struct dentry*
vfsub_lock_rename(struct dentry *d1, struct dentry *d2)
{
	struct dentry *d;
	lockdep_off();
	d = lock_rename(d1, d2);
	lockdep_on();
	return d;
}

static inline void vfsub_unlock_rename(struct dentry *d1, struct dentry *d2)
{
	lockdep_off();
	unlock_rename(d1, d2);
	lockdep_on();
}

static inline struct file*
vfsub_filp_open(const char *path, int oflags, int mode)
{
	struct file *err;
	lockdep_off();
	err = filp_open(path, oflags, mode);
	lockdep_on();
	return err;
}

static inline int
vfsub_mkdir(struct inode *dir, struct dentry *dentry, int mode)
{
	int err;
	lockdep_off();
	err = vfs_mkdir(dir, dentry, mode);
	lockdep_on();
	return err;
}

#endif /* __KERNEL__ */
#endif /* __AUFS_VFSUB_H__ */
